/* This file is part of slogin
   Copyright (C) 2000,2001,2004 Sergey Poznyakoff
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

static char rcsid[] =
"$Id: slogin.c,v 1.2 2004/05/27 07:31:25 gray Exp $";

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <sys/types.h>
#include <sys/time.h>

#include <stdio.h>
#include <stdlib.h>
#include <pwd.h>
#include <time.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <sys/wait.h>
#include <unistd.h>
#include <utmp.h>

#if defined(HAVE_SHADOW_H)
# include <shadow.h>
#endif /* HAVE_SHADOW_H */

#if defined(HAVE_CRYPT_H)
# include <crypt.h>
#endif

#ifdef OSFC2
# include <sys/security.h>
# include <prot.h>
#endif
#include <slogin.h>

#define DEFAULT_SHELL "/bin/sh"

/*
 *	Check the users password against the standard UNIX
 *	password table.
 */
char *
unix_pass(char *name, char *passwd)
{
	struct passwd	*pwd;
	char		*encpw;
	char		*encrypted_pass;
#if defined(HAVE_SHADOW_H)
#if defined(M_UNIX)
	struct passwd	*spwd;
#else
	struct spwd	*spwd;
#endif
#endif /* HAVE_SHADOW_H */

#ifdef OSFC2
	struct pr_passwd *pr_pw;

	if ((pr_pw = getprpwnam(name)) == NULL)
		return NULL;
	encrypted_pass = pr_pw->ufld.fd_encrypt;
#else /* OSFC2 */
	/*
	 *	Get encrypted password from password file
	 */
	if ((pwd = getpwnam(name)) == NULL) {
		return NULL;
	}
	encrypted_pass = pwd->pw_passwd;
#endif /* OSFC2 */

#if defined(HAVE_SHADOW_H)
	/*
	 *      See if there is a shadow password.
	 */
	if ((spwd = getspnam(name)) != NULL)
#if defined(M_UNIX)
		encrypted_pass = spwd->pw_passwd;
#else
		encrypted_pass = spwd->sp_pwdp;
#endif	/* M_UNIX */
#endif	/* HAVE_SHADOW_H */

	/*
	 *	We might have a passwordless account.
	 */
	if (encrypted_pass[0] == 0)
		return pwd->pw_shell;

	/*
	 *	Check encrypted password.
	 */
	encpw = crypt(passwd, encrypted_pass);
	if (strcmp(encpw, encrypted_pass))
		return NULL;

	setgid(pwd->pw_gid);
	setuid(pwd->pw_uid);
	
	return pwd->pw_shell;
}

#define MAX_USERNAME sizeof(((struct utmp*)0)->ut_user)

int
main(int argc, char **argv)
{
	char *user;
	char username[MAX_USERNAME];
	char *pwd;
	char *shell;

	do {
		user = "root";
		pwd = slogin_getpass("root password (or ^D to log in as a user): ");
		if (!pwd) {
			/* Try to log in as a regular user */
			user = slogin_getuser("login: ");
			if (!user)
				continue;
			memcpy(username, user, sizeof(username));
			username[sizeof(username)-1] = 0;
			user = username;
			pwd = slogin_getpass("password: ");
		}
	} while (!(shell = unix_pass(user, pwd)));
	execl(shell, shell, NULL);
	execl(DEFAULT_SHELL, DEFAULT_SHELL, NULL);
	/*NOTREACDHED*/
}


